==> 0.4.2&file=in_ffsox-2a-0.4.2-src.tar.gz: in_ffsox-2a-0.4.2/libffsox/ffsox.h <==
/*
 * ffsox.h
 * Copyright (C) 2010 - 2014 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __FFSOX_H__
#define __FFSOX_H__

#include <pb.h>
#define LIBFFSOX_BS1770
#if defined (LIBFFSOX_BS1770) // [
#include <bs1770.h>
#endif // ]

#if defined (__FFSOX_PROTO__) // ]
#include <lame/lame.h>
#include <libavutil/opt.h>
#include <libavformat/avformat.h>
#else // ] [
#define avpicture_deinterlace DEPRECATED_avpicture_deinterlace
#include "ffsox_sym.h"
#if 0 // [
#undef avpicture_deinterlace
int avpicture_deinterlace(AVPicture *dst, const AVPicture *src,
                          enum AVPixelFormat pix_fmt, int width, int height);
#endif // ]
#endif // ]

//#define FFSOX_DEBUG
#define FFSOX_VST
#define FFSOX_AUDIO_STEREO
#define FFSOX_AUDIO_DRC
#define FFSOX_AUDIO_CUST

#define FFSOX_ZERO(p) \
  (memset(p,0,sizeof *(p)))

///////////////////////////////////////////////////////////////////////////////
typedef int16_t ffsox_s16_t;
typedef int32_t ffsox_s32_t;
typedef float ffsox_f32_t;
typedef double ffsox_f64_t;

///////////////////////////////////////////////////////////////////////////////
//#define FFO_CLONE_STREAM_META
#if defined (FFO_CLONE_STREAM_META) // {
#define ffi_metadata(ffi) ( \
  NULL!=(ffi)->ic->metadata?(ffi)->ic->metadata: \
  NULL!=(ffi)->ast?(ffi)->ast->st->metadata: \
  NULL \
)
#else // } {
#define ffi_metadata(ffi) ( \
  NULL!=(ffi)->ic->metadata?(ffi)->ic->metadata: \
  NULL \
)
#endif // }

enum {
  FFI_CSV=1<<0,
#if ! defined (FFO_CLONE_STREAM_META) // [
  FFI_STREAM_META=1<<1,
#endif // ]
  FFI_OPEN=1<<2,
#if defined (FFSOX_AUDIO_STEREO) // [
  FFI_AUDIO_STEREO=1<<3,
#else // ] [
  FFI_AUDIO_STEREO=0,
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
  FFI_AUDIO_DRC=1<<4,
#else // ] [
  FFI_AUDIO_DRC=0,
#endif // ]
  FFI_DEFAULT=FFI_CSV|FFI_STREAM_META|FFI_OPEN|FFI_AUDIO_STEREO
};

typedef struct ffi_st {
  AVStream *st;
#if ! defined (FFSOX_AUDIO_CUST) && defined (FFSOX_AUDIO_STEREO) // [
  int mode;
#endif // ]
  AVCodec *dec;
  AVCodecContext *dc;
  int index;
  ////
  int decoding_needed;
  int saw_first_ts;
  int64_t dts;
  int64_t next_dts;
  ////
  int64_t pts;
  int64_t next_pts;
} ffi_st_t;

#if /*! defined (FFSOX_AUDIO_CUST) &&*/ defined (FFSOX_AUDIO_DRC) // [
ffi_st_t *ffi_st_init(ffi_st_t *ffi_st, AVStream *st, int mode, double *scale);
#else // ] [
ffi_st_t *ffi_st_init(ffi_st_t *ffi_st, AVStream *st, int mode);
#endif // ]
ffi_st_t *ffi_st_close(ffi_st_t *ffi_st);

typedef struct ffi {
  AVFormatContext *ic;
  int64_t ts_offset;
  int nb_streams;
  ffi_st_t *st;
  ffi_st_t *ast;
#if defined (FFSOX_VST) // {
  ffi_st_t *vst;
#endif // }
} ffi_t;

#if /*! defined (FFSOX_AUDIO_CUST) &&*/ defined (FFSOX_AUDIO_DRC) // [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, int mode, double *scale);
#else // ] [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, int mode);
#endif // ]
ffi_t *ffi_cleanup(ffi_t *ffi);

///////////////////////////////////////////////////////////////////////////////
#if /*! defined (FFSOX_AUDIO_CUST) &&*/ defined (FFSOX_AUDIO_DRC) // [
int ffsox_clone(pb_str_t *fn1, pb_str_t *fn2, int mode, char **meta,
    double *scale);
#else // ] [
int ffsox_clone(pb_str_t *in, pb_str_t *out, int mode, char **meta);
#endif // ]
int ffsox_lame(pb_str_t *in, pb_str_t *out, int mode, double gain,
    double quality);

///////////////////////////////////////////////////////////////////////////////
#define ffsox_ar_valid(ar) \
  (FFSOX_AR_VALID&(ar)->state)
#define ffsox_ar_frame(ar) \
  ((ar)->cbs->frame((ar)->data))
#define ffsox_ar_stream(ar) \
  ((ar)->cbs->stream((ar)->data))
#define ffsox_ar_read(ar,pkt) \
  ((ar)->cbs->read((ar)->data,pkt))
#define ffsox_ar_got_frame(ar,frame) \
  ((ar)->cbs->got_frame((ar)->data,ar,frame))
#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
#define ffsox_ar_stereo(ar) \
  ((ar)->cbs->stereo((ar)->data))
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
#define ffsox_ar_drc_enabled(ar) \
  ((ar)->cbs->drc.enabled((ar)->data))
#define ffsox_ar_drc_scale(ar) \
  ((ar)->cbs->drc.scale((ar)->data))
#endif // ]
#endif // ]

typedef struct ffsox_ar ffsox_ar_t;
typedef struct ffsox_ar_cbs ffsox_ar_cbs_t;

enum {
  FFSOX_AR_RUN=1,
  FFSOX_AR_FLUSH=FFSOX_AR_RUN<<1,
  FFSOX_AR_DONE=FFSOX_AR_FLUSH<<1,
  FFSOX_AR_ERROR=FFSOX_AR_DONE<<1,
  FFSOX_AR_VALID=FFSOX_AR_RUN|FFSOX_AR_FLUSH
};

struct ffsox_ar_cbs {
  AVFrame *(*frame)(void *data);
  AVStream *(*stream)(void *data);
  int (*read)(void *data, AVPacket *pkt);
  void (*got_frame)(void *data, ffsox_ar_t *ar, AVFrame *frame);
#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
  int (*stereo)(void *data);
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
  struct {
    int (*enabled)(void *data);
    double (*scale)(void *data);
  } drc;
#endif // ]
#endif // ]
};

struct ffsox_ar {
  void *data;
  ffsox_ar_cbs_t *cbs;

#if 0 && defined (FFSOX_AUDIO_CUST) // [
  ffi_st_t *st;
#endif // ]
  double q;
  int count;
  AVPacket pkt1;
  AVPacket pkt2;
  AVFrame *frame;
  int state;
#if defined (LIBFFSOX_BS1770) // [
  bs1770_lufs_t *lufs;
#endif // ]
};

#if 0 && defined (FFSOX_AUDIO_CUST) // [
ffsox_ar_t *ffsox_ar_init(ffsox_ar_t *ar, double q, ffi_st_t *st,
    void *data, ffsox_ar_cbs_t *cbs);
ffsox_ar_t *ffsox_ar_init_ffi(ffsox_ar_t *ar, double q, ffi_st_t *st,
    ffi_t *ffi);
#else // ] [
ffsox_ar_t *ffsox_ar_init(ffsox_ar_t *ar, double q, void *data,
    ffsox_ar_cbs_t *cbs);
ffsox_ar_t *ffsox_ar_init_ffi(ffsox_ar_t *ar, double q, ffi_t *ffi);
#endif // ]
#if defined (LIBFFSOX_BS1770) // [
void ffsox_ar_reset(ffsox_ar_t *ar, double q, bs1770_lufs_t *lufs);
#else // ] [
void ffsox_ar_reset(ffsox_ar_t *ar, double q);
#endif // ]
ffsox_ar_t *ffsox_ar_cleanup(ffsox_ar_t *ar);
void ffsox_ar_next(ffsox_ar_t *ar);

///////////////////////////////////////////////////////////////////////////////
#define ffsox_buf_avail(b) \
  ((b)->mp-(b)->wp)

typedef struct ffsox_buf {
  uint8_t *rep;
  uint8_t *wp;
  uint8_t *mp;
} ffsox_buf_t;

ffsox_buf_t *ffsox_buf_init(ffsox_buf_t *buf, size_t size);
ffsox_buf_t *ffsox_buf_cleanup(ffsox_buf_t *buf);

ffsox_buf_t *ffsox_buf_realloc(ffsox_buf_t *buf, size_t len);

///////////////////////////////////////////////////////////////////////////////
sox_effect_handler_t const *ffsox_sox_reader();

#endif // __FSOX_H__

==> 0.6.0&file=in_ffsox-2a-0.6.0-src.tar.gz: in_ffsox-2a-0.6.0/libffsox/ffsox.h <==
/*
 * ffsox.h
 * Copyright (C) 2010 - 2014 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __FFSOX_H__
#define __FFSOX_H__

#include <pb.h>
//#define LIBFFSOX_BS1770
#if defined (LIBFFSOX_BS1770) // [
#include <bs1770.h>
#endif // ]

//#if defined (__FFSOX_PROTO__) // ]
//#include <lame/lame.h>
#include <libavutil/opt.h>
#include <libavformat/avformat.h>
#include <libswresample/swresample.h>
#include <libavfilter/avfilter.h>
#include <sox.h>
//#else // ] [
//#include "ffsox_sym.h"
//#endif // ]

#define FFSOX_VST
#define FFSOX_AUDIO_STEREO
#define FFSOX_AUDIO_DRC
#define FFSOX_AUDIO_CUST
#define FFSOX_EXPERIMENTAL
//#define FFSOX_FILTER_AR
#if defined (FFSOX_EXPERIMENTAL) // [
#define FFSOX_MAP_MEMORY
#endif // ]

#define FFSOX_DEPRECATED
#if defined (FFSOX_DEPRECATED) // [
#define FFSOX_DEPRECATED_FREE_PACKET
#if defined (_WIN32) // [
//#define FFSOX_AR_DELAY
#endif // ]

#define FFSOX_DEPRECATED_STREAM_CODEC
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
#define FFSOX_DEPRECATED_DECODE_VIDEO2
#define FFSOX_DEPRECATED_PICTURE_CROP
#define FFSOX_DEPRECATED_AV_FORMAT_CONTEXT_FILENAME
#endif // ]
#endif // ]

#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
//#define FFSOX_FFI_DECODE
#endif // ]

// Not really needed because FFSOX_LOCAL_AR is un-defined.
#if defined (FFSOX_FFI_DECODE) && defined (FFSOX_LOCAL_AR) // [
#undef FFSOX_LOCAL_AR
#endif // ]

#define FFSOX_ZERO(p) \
  (memset(p,0,sizeof *(p)))

///////////////////////////////////////////////////////////////////////////////
typedef int16_t ffsox_s16_t;
typedef int32_t ffsox_s32_t;
typedef float ffsox_f32_t;
typedef double ffsox_f64_t;

///////////////////////////////////////////////////////////////////////////////
//#define FFO_CLONE_STREAM_META
#if defined (FFO_CLONE_STREAM_META) // {
#define ffi_metadata(ffi) ( \
  NULL!=(ffi)->ic->metadata?(ffi)->ic->metadata: \
  NULL!=(ffi)->ast?(ffi)->ast->st->metadata: \
  NULL \
)
#else // } {
#define ffi_metadata(ffi) ( \
  NULL!=(ffi)->ic->metadata?(ffi)->ic->metadata: \
  NULL \
)
#endif // }

#if defined (FFSOX_FFI_DECODE) // [
#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
#define ffi_st_stereo(ffi_st) \
  ((ffi_st)->cbs->stereo((ffi_st)->data))
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
#define ffi_st_drc_enabled(ffi_st) \
  ((ffi_st)->cbs->drc.enabled((ffi_st)->data))
#define ffi_st_drc_scale(ffi_st) \
  ((ffi_st)->cbs->drc.scale((ffi_st)->data))
#endif // ]
#endif // ]

typedef struct ffi_st ffi_st_t;

typedef struct ffi_st_cbs {
#if defined (FFSOX_AUDIO_STEREO) // [
  int (*stereo)(void *data);
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
  struct {
    int (*enabled)(void *data);
    double (*scale)(void *data);
  } drc;
#endif // ]
} ffi_st_cbs_t;
#endif // ]

enum {
  FFI_CSV=1<<0,
#if ! defined (FFO_CLONE_STREAM_META) // [
  FFI_STREAM_META=1<<1,
#endif // ]
  FFI_OPEN=1<<2,
#if defined (FFSOX_FFI_DECODE) // [
  FFI_DEFAULT=FFI_CSV|FFI_STREAM_META|FFI_OPEN
#else // ] [
#if defined (FFSOX_AUDIO_STEREO) // [
  FFI_AUDIO_STEREO=1<<3,
#else // ] [
  FFI_AUDIO_STEREO=0,
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
  FFI_AUDIO_DRC=1<<4,
#else // ] [
  FFI_AUDIO_DRC=0,
#endif // ]
  FFI_DEFAULT=FFI_CSV|FFI_STREAM_META|FFI_OPEN|FFI_AUDIO_STEREO
#endif // ]
};

typedef struct ffi_st {
#if defined (FFSOX_FFI_DECODE) // [
  ffi_st_cbs_t *cbs;
  void *data;
#endif // ]
  AVStream *st;
#if ! defined (FFSOX_FFI_DECODE) // [
#if ! defined (FFSOX_AUDIO_CUST) && defined (FFSOX_AUDIO_STEREO) // [
  int mode;
#endif // ]
#endif // ]
  AVCodec *dec;
  AVCodecContext *dc1;
  int index;
  ////
  int decoding_needed;
  int saw_first_ts;
  int64_t dts;
  int64_t next_dts;
  ////
  int64_t pts;
  int64_t next_pts;
} ffi_st_t;

#if defined (FFSOX_FFI_DECODE) // [
ffi_st_t *ffi_st_init(ffi_st_t *ist, AVStream *st, int mode,
    ffi_st_cbs_t *cbs, void *data);
#elif /*! defined (FFSOX_AUDIO_CUST) &&*/ defined (FFSOX_AUDIO_DRC) // [
ffi_st_t *ffi_st_init(ffi_st_t *ist, AVStream *st, int mode, double *scale);
#else // ] [
ffi_st_t *ffi_st_init(ffi_st_t *ist, AVStream *st, int mode);
#endif // ]
ffi_st_t *ffi_st_close(ffi_st_t *ist);

typedef struct ffi {
#if defined (FFSOX_MAP_MEMORY) // [
  size_t avio_ctx_buffer_size;

  struct {
    size_t size;
    uint8_t *rep;
    uint8_t *mp;
    uint8_t *rp;
  } map;

  AVIOContext *avio_ctx;
#endif // ]
  AVFormatContext *ic;
  int64_t ts_offset;
  int nb_streams;
  ffi_st_t *ist;
  ffi_st_t *ast;
#if defined (FFSOX_VST) // {
  ffi_st_t *vst;
#endif // }
} ffi_t;

AVCodec *ffi_find_decoder(enum AVCodecID id);
#if defined (FFSOX_MAP_MEMORY) // [
#if defined (FFSOX_FFI_DECODE) // [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, size_t avio_ctx_buffer_size,
    int mode, ffi_st_cbs_t *cbs, void *data);
#elif defined (FFSOX_AUDIO_DRC) // ] [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, size_t avio_ctx_buffer_size,
    int mode, double *scale);
#else // ] [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, size_t avio_ctx_buffer_size,
    int mode);
#endif // ]
#else // ] [
#if defined (FFSOX_FFI_DECODE) // [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, int mode, ffi_st_cbs_t *cbs,
    void *data);
#elif defined (FFSOX_AUDIO_DRC) // [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, int mode, double *scale);
#else // ] [
ffi_t *ffi_init(ffi_t *ffi, pb_str_t *fn, int mode);
#endif // ]
#endif // ]
ffi_t *ffi_cleanup(ffi_t *ffi);

///////////////////////////////////////////////////////////////////////////////
#if defined (FFSOX_FFI_DECODE) // [
int ffsox_clone(pb_str_t *fn1, pb_str_t *fn2, int mode, char **meta,
    ffi_st_cbs_t *cbs, void *data);
#elif defined (FFSOX_AUDIO_DRC) // ] [
int ffsox_clone(pb_str_t *fn1, pb_str_t *fn2, int mode, char **meta,
    double *scale);
#else // ] [
int ffsox_clone(pb_str_t *in, pb_str_t *out, int mode, char **meta);
#endif // ]

int ffsox_lame(pb_str_t *in, pb_str_t *out, int mode, double gain,
    double quality);

///////////////////////////////////////////////////////////////////////////////
#if defined (FFSOX_FILTER_AR) // [
typedef struct ffsox_audio_filter {
  AVFilterGraph *pFilterGraph;
  AVFrame *pFrame;
} ffsox_audio_filter_t;
#endif // ]

///////////////////////////////////////////////////////////////////////////////
#define ffsox_ar_valid(ar) \
  (FFSOX_AR_VALID&(ar)->state)
#define ffsox_ar_frame(ar) \
  ((ar)->cbs->frame((ar)->data))
#define ffsox_ar_stream(ar) \
  ((ar)->cbs->stream((ar)->data))
#define ffsox_ar_read(ar,pkt) \
  ((ar)->cbs->read((ar)->data,pkt))
#if ! defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
#define ffsox_ar_got_frame(ar,frame) \
  ((ar)->cbs->got_frame((ar)->data,ar,frame))
#endif // ]
#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
#define ffsox_ar_stereo(ar) \
  ((ar)->cbs->stereo((ar)->data))
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
#define ffsox_ar_drc_enabled(ar) \
  ((ar)->cbs->drc.enabled((ar)->data))
#define ffsox_ar_drc_scale(ar) \
  ((ar)->cbs->drc.scale((ar)->data))
#endif // ]
#endif // ]

typedef struct ffsox_ar ffsox_ar_t;
typedef struct ffsox_ar_cbs ffsox_ar_cbs_t;

enum {
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
  FFSOX_AR_READ_PACKET=1,
  FFSOX_AR_RECEIVE_FRAME=FFSOX_AR_READ_PACKET<<1,
#if defined (FFSOX_AUDIO_STEREO) // [
  FFSOX_AR_CONVERT_FRAME=FFSOX_AR_RECEIVE_FRAME<<1,
  FFSOX_AR_FLUSH=FFSOX_AR_CONVERT_FRAME<<1,
#else // ] [
  FFSOX_AR_FLUSH=FFSOX_AR_RECEIVE_FRAME<<1,
#endif // ]
#else // ] [
  FFSOX_AR_RUN=1,
  FFSOX_AR_FLUSH=FFSOX_AR_RUN<<1,
#endif // ]
  FFSOX_AR_DONE=FFSOX_AR_FLUSH<<1,
  FFSOX_AR_ERROR=FFSOX_AR_DONE<<1,
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
  FFSOX_AR_MASK=FFSOX_AR_READ_PACKET
      |FFSOX_AR_RECEIVE_FRAME
#if defined (FFSOX_AUDIO_STEREO) // [
      |FFSOX_AR_CONVERT_FRAME
#endif // ]
      ,
  FFSOX_AR_VALID=FFSOX_AR_RECEIVE_FRAME
#if defined (FFSOX_AUDIO_STEREO) // [
      |FFSOX_AR_CONVERT_FRAME
#endif // ]
      ,
#else // ] [
  FFSOX_AR_VALID=FFSOX_AR_RUN|FFSOX_AR_FLUSH,
#endif // ]
};

struct ffsox_ar_cbs {
  AVFrame *(*frame)(void *data);
  AVStream *(*stream)(void *data);
  int (*read)(void *data, AVPacket *pkt);
  void (*got_frame)(void *data, ffsox_ar_t *ar, AVFrame *frame);
#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
  int (*stereo)(void *data);
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
  struct {
    int (*enabled)(void *data);
    double (*scale)(void *data);
  } drc;
#endif // ]
#endif // ]
};

struct ffsox_ar {
  ffsox_ar_cbs_t *cbs;
  void *data;

  double q;
  int count;
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
  AVPacket pkt;
#else // ] [
  AVPacket pkt1;
  AVPacket pkt2;
#endif // ]
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
  struct {
    AVFrame *frame;
  } decode;

#if defined (FFSOX_AUDIO_STEREO) // [
  struct {
    SwrContext *swr_ctx;
    AVFrame *frame;
  } convert;
#endif // ]

  ffi_st_t *ist;
#else // ] [
  AVFrame *frame;
#endif // ]
  int state;
#if defined (LIBFFSOX_BS1770) // [
  bs1770_lufs_t *lufs;
#endif // ]
#if defined (FFSOX_AR_DELAY) // [
	struct {
		union {
			FILETIME ft;
			uint64_t ui64;
		} start;

		union {
			FILETIME ft;
			uint64_t ui64;
		} end;
	} stopwatch;
#endif // ]
#if defined (FFSOX_FILTER_AR) // [
  ffsox_audio_filter_t filter;
#endif // ]
};

#if 0 && defined (FFSOX_AUDIO_CUST) // [
ffsox_ar_t *ffsox_ar_init(ffsox_ar_t *ar, double q, ffi_st_t *st,
    void *data, ffsox_ar_cbs_t *cbs);
ffsox_ar_t *ffsox_ar_init_ffi(ffsox_ar_t *ar, double q, ffi_st_t *st,
    ffi_t *ffi);
#else // ] [
ffsox_ar_t *ffsox_ar_init(ffsox_ar_t *ar, double q, void *data,
    ffsox_ar_cbs_t *cbs);
ffsox_ar_t *ffsox_ar_init_ffi(ffsox_ar_t *ar, double q, ffi_t *ffi);
#endif // ]
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
#if defined (LIBFFSOX_BS1770) // [
void ffsox_ar_reset(ffsox_ar_t *ar, ffi_st_t *ist, double q,
    bs1770_lufs_t *lufs);
#else // ] [
void ffsox_ar_reset(ffsox_ar_t *ar, ffi_st_t *ist, double q);
#endif // ]
#else // ] [
#if defined (LIBFFSOX_BS1770) // [
void ffsox_ar_reset(ffsox_ar_t *ar, double q, bs1770_lufs_t *lufs);
#else // ] [
void ffsox_ar_reset(ffsox_ar_t *ar, double q);
#endif // ]
#endif // ]
ffsox_ar_t *ffsox_ar_cleanup(ffsox_ar_t *ar);
AVFrame *ffsox_ar_frame_internal(ffsox_ar_t *ar);
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
void ffsox_ar_got_frame(ffsox_ar_t *ar, AVFrame *frame);
#endif // ]
void ffsox_ar_next(ffsox_ar_t *ar);

///////////////////////////////////////////////////////////////////////////////
#define ffsox_buf_avail(b) \
  ((b)->mp-(b)->wp)

typedef struct ffsox_buf {
  uint8_t *rep;
  uint8_t *wp;
  uint8_t *mp;
} ffsox_buf_t;

ffsox_buf_t *ffsox_buf_init(ffsox_buf_t *buf, size_t size);
ffsox_buf_t *ffsox_buf_cleanup(ffsox_buf_t *buf);

ffsox_buf_t *ffsox_buf_realloc(ffsox_buf_t *buf, size_t len);

///////////////////////////////////////////////////////////////////////////////
sox_effect_handler_t const *ffsox_sox_reader();

#endif // __FSOX_H__
